import { BasePlugin } from '@playkit-js/kaltura-player-js';
import { KalturaPlayer } from '@playkit-js/kaltura-player-js';

export declare const EventType: {
    [event: string]: string;
};

export declare const HttpMethodType: KavaHttpMethodType;

/**
 * @typedef {object} KavaConfigobject@typedef {object} KavaConfigobject
 * @property {string} [serviceUrl] - The Kaltura API server.
 * @property {number} [viewEventCountdown] - The interval in seconds that VIEW event will be sent.
 * @property {number} [resetSessionCountdown] - The interval in seconds that Kava session will be reset.
 * @property {number} [dvrThreshold] - Threshold in seconds from the live edge.
 * @property {string} [application] - Used to send the Player application playing the entry.
 * @property {string} [kalturaApplicationVersion] - KalturaApplication Used to send the kaltura application name KalturaApplication:(KMC,KMS,KAF,PITCH,KMS_GO,WEBCASTING,PERSONAL_CAPTURE,KME,KME_MOBILE_APP,PATH,LECTURE_CAPTURE,EP,CHAT_AND_COLLABORATION,ANALYTICS,EXPRESS_CAPTURE,KPF,KAF_GO,SITES,STUDIO
 ) from which the user is playing the entry.
 * @property {string} [kalturaApplication] - Used to send the kaltura application version from which the user is playing the entry.
 * @property {string} [playbackContext] - Used to send the id of the category from which the user is playing the entry.
 * @property {Function} [tamperAnalyticsHandler] - An optional handler to implement. Can be used to manipulate the model data before analytics event sent, or to cancel a certain analytics request.
 * @property {object} [customVar1] - Custom objects field.
 * @property {object} [customVar2] - Custom objects field.
 * @property {object} [customVar3] - Custom objects field.
 * @property {string} [userId] - custom user id .
 * @property {string} [hostingKalturaApplication] - Used to send the application from which the user is playing the entry.
 * @property {string} [hostingKalturaApplicationVersion] - Used to send the application version from which the user is playing the entry.
 * @example
 * // Default config
 * {
 *  serviceUrl: '//analytics.kaltura.com/api_v3/index.php',
 *  viewEventCountdown: 30,
 *  resetSessionCountdown: 30,
 *  dvrThreshold: 120,
 *  application: '',
 *  kalturaApplicationVersion: '',
 *  kalturaApplication: ''
 *  playbackContext: '',
 *  userId: '',
 *  hostingKalturaApplication: '',
 *  hostingKalturaApplicationVersion: ''
 * }
 */
declare type KavaConfigObject = {
    serviceUrl?: string;
    tamperAnalyticsHandler?: Function;
    viewEventCountdown?: number;
    resetSessionCountdown?: number;
    dvrThreshold?: number;
    customVar1?: object;
    customVar2?: object;
    customVar3?: object;
    application?: string;
    kalturaApplicationVersion?: string;
    kalturaApplication?: string;
    playbackContext?: string;
    userId?: string;
    hostingKalturaApplication?: string;
    hostingKalturaApplicationVersion?: string;
};

declare type KavaHttpMethodType = {
    [stream: string]: string;
};

export declare const NAME: string;

/**
 * Kaltura Advanced Analytics plugin.
 * @class Kava
 * @param {string} name - The plugin name.
 * @param {Player} player - The player instance.
 * @param {KavaConfigObject} config - The plugin config.
 */
declare class Plugin_2 extends BasePlugin {
    private _model;
    private _timer;
    private _rateHandler;
    private _viewEventEnabled;
    private _firstPlayRequestTime;
    private _bufferStartTime;
    private _previousCurrentTime;
    private _isFirstPlay;
    private _isFirstPlaying;
    private _isEnded;
    private _isPaused;
    private _isBuffering;
    private _timePercentEvent;
    private _isPlaying;
    private _loadStartTime;
    private _lastDroppedFrames;
    private _lastTotalFrames;
    private _performanceObserver;
    private _performanceEntries;
    private _pendingFragLoadedUrls;
    private _fragLoadedFiredOnce;
    private _canPlayOccured;
    private _isManualPreload;
    private _lastViewEventPlayTime;
    /**
     * Default config of the plugin.
     * @type {Object}
     * @static
     * @memberof Kava
     */
    static defaultConfig: any;
    /**
     * @static
     * @return {boolean} - Whether the plugin is valid in the current environment.
     * @memberof Kava
     */
    static isValid(): boolean;
    constructor(name: string, player: KalturaPlayer, config: KavaConfigObject);
    private _updateSoundModeInModel;
    private _updateViewabilityModeInModel;
    private _handleNewPerformanceEntries;
    /**
     * Destroys the plugin.
     * @return {void}
     * @memberof Kava
     * @instance
     */
    destroy(): void;
    private _reset;
    /**
     * Reset the plugin.
     * @return {void}
     * @memberof Kava
     * @instance
     */
    reset(): void;
    /**
     * loadMedia of the plugin.
     * @return {void}
     * @memberof Kava
     * @instance
     */
    loadMedia(): void;
    /**
     * Gets the model object for a certain event.
     * @param {string} event - Event name.
     * @returns {Object} - Model object.
     * @instance
     * @memberof Kava
     * @example
     * const kava = player.plugins.kava;
     * const viewModel = kava.getEventModel(kava.EventType.VIEW);
     * kava.sendAnalytics(viewModel);
     */
    getEventModel(event: string): any;
    /**
     * @returns {KavaEventType} - The kava events list.
     * @instance
     * @memberof Kava
     */
    get EventType(): {
        [event: string]: string;
    };
    /**
     * Sends KAVA analytics event to analytics service.
     * @param {Object} model - Event model.
     * @returns {Promise} - Promise to indicate request succeed or failed.
     * @instance
     * @memberof Kava
     * @example
     * player.plugins.kava.sendAnalytics({...})
     * .then(() => {
     *   console.log('kava analytics sent successfully');
     * })
     * .catch(e => {
     *   console.log('kava analytics send failed', e);
     * });
     */
    sendAnalytics(model: any): Promise<void>;
    private _resetFlags;
    private _resetSession;
    private _sendAnalytics;
    private _handleServerResponseSuccess;
    private _handleServerResponseFailed;
    private _addBindings;
    private _bindPlaykitUIEvents;
    private _bindApplicationEvents;
    private _onFirstPlaying;
    private _onLoadStart;
    private _getRates;
    private _setInitialTracks;
    /**
     * gets the available buffer length
     * @returns {number} the remaining buffer length of the current played time range
     * @private
     */
    private _getAvailableBuffer;
    /**
     * calculates the forward buffer health ratio
     * @returns {number} the ratio between available buffer and the target buffer
     * @private
     */
    private _getForwardBufferHealth;
    /**
     * get the target buffer length from the player
     * @returns {number} the target buffer in seconds
     * @private
     */
    private _getTargetBuffer;
    /**
     * calculates the dropped frames ratio since last call
     * @returns {number} the ratio between dropped frames and the total frames
     * @private
     */
    private _getDroppedFramesRatio;
    /**
     * returns dropped and total frames from the VideoPlaybackQuality Interface of the browser (if supported)
     * @returns {number} {number} the number of video frames dropped and total video frames (created and dropped)
     * since the creation of the associated HTMLVideoElement
     * @private
     */
    private _getDroppedAndDecodedFrames;
    private _onReport;
    private _updateNetworkConnectionTypeinModel;
    private _initNetworkConnectionType;
    private _onPlaying;
    private _onCanPlay;
    private _onFirstPlay;
    private _onSourceSelected;
    private _onSeeking;
    private _onPause;
    private _onEnded;
    private _onTimeUpdate;
    private _onFragLoaded;
    private _handleFragPerformanceObserver;
    private _updateMaxNetworkConnectionOverhead;
    private _updateFragLoadedStats;
    private _onManifestLoaded;
    private _onTimedMetadataLoaded;
    private _onVideoTrackChanged;
    private _onAudioTrackChanged;
    private _onTextTrackChanged;
    private _onError;
    private _onPlaybackRateChanged;
    private _onPlayerStateChanged;
    private _onRelatedClicked;
    private _onRelatedSelected;
    private _onShareClicked;
    private _onShareNetworkClicked;
    private _onReportSubmitted;
    private _onReportClicked;
    private _onInfoScreenOpened;
    private _onDownloadItemClicked;
    private _onFullScreenChanged;
    private _updateSessionStartTimeModel;
    private _updateBufferModel;
    private _updatePlayTimeSumModel;
    private _setModelDelegates;
    private _getPlayerType;
    private _getKalturaApplicationId;
    private _getPosition;
    private _getDeliveryType;
    private _getPlaybackType;
    private _validate;
    private _isApplicationEventValid;
    private _logMissingParam;
    private static _getTimeDifferenceInSeconds;
    private _updateTabModeInModel;
    private _isTabHidden;
    private _initTabMode;
}
export { Plugin_2 as Plugin }

export declare const VERSION: string;

export { }
